@@ -2,37 +2,40 @@
 # This file is part of Quark-Engine - https://github.com/quark-engine/quark-engine
 # See the file 'LICENSE' for copying permission.
 
-import json
-import os
-
 import click
+import json
+import numpy as np
+import os
 from tqdm import tqdm
 
+from quark import __version__
 from quark import config
-from quark.Objects.quark import Quark
-from quark.Objects.quarkrule import QuarkRule
+from quark.core.quark import Quark
+from quark.core.struct.ruleobject import RuleObject
 from quark.logo import logo
-from quark.utils.out import print_success, print_info, print_warning
-from quark.utils.weight import Weight
 from quark.utils.colors import yellow
+from quark.utils.graph import show_comparison_graph, select_label_menu
+from quark.utils.pprint import print_success, print_info, print_warning
+from quark.utils.weight import Weight
 
 logo()
 
 
+@click.version_option(version=__version__)
 @click.command(no_args_is_help=True)
 @click.option(
     "-s",
     "--summary",
     is_flag=False,
     flag_value="all_rules",
-    help="Show summary report",
+    help="Show summary report. Optionally specify the name of a rule/label",
 )
 @click.option(
     "-d",
     "--detail",
     is_flag=False,
     flag_value="all_rules",
-    help="Show detail report",
+    help="Show detail report. Optionally specify the name of a rule/label",
 )
 @click.option(
     "-o",
@@ -47,6 +50,7 @@ logo()
     help="APK file",
     type=click.Path(exists=True, file_okay=True, dir_okay=False),
     required=True,
+    multiple=True,
 )
 @click.option(
     "-r",
@@ -74,7 +78,7 @@ logo()
 @click.option(
     "-t",
     "--threshold",
-    help="Set the confidence threshold",
+    help="Set the lower limit of the confidence threshold",
     type=click.Choice(["100", "80", "60", "40", "20"]),
     required=False,
 )
@@ -99,6 +103,21 @@ logo()
     type=click.Choice(["max", "detailed"]),
     required=False,
 )
+@click.option(
+    "-C",
+    "--comparison",
+    help="Behaviors comparison based on max confidence of rule labels",
+    required=False,
+    is_flag=True,
+)
+@click.option(
+    "--core-library",
+    "core_library",
+    help="Specify the core library used to analyze an APK",
+    type=click.Choice(("androguard", "rizin"), case_sensitive=False),
+    required=False,
+    default="androguard",
+)
 def entry_point(
     summary,
     detail,
@@ -111,14 +130,82 @@ def entry_point(
     list,
     permission,
     label,
+    comparison,
+    core_library,
 ):
     """Quark is an Obfuscation-Neglect Android Malware Scoring System"""
 
     # Load APK
-    data = Quark(apk)
+    data = Quark(apk[0], core_library)
 
     # Load rules
-    rules_list = [x for x in os.listdir(rule) if x.endswith("json")]
+    rules_list = [file for file in os.listdir(rule) if file.endswith("json")]
+
+    if comparison:
+
+        # selection of labels on which it will be done the comparison on radar chart
+        # first look for all label found in the rule list
+        all_labels = set()
+        for single_rule in tqdm(rules_list):
+            rulepath = os.path.join(rule, single_rule)
+            rule_checker = RuleObject(rulepath)
+            labels = rule_checker.label  # array type, e.g. ['network', 'collection']
+            for single_label in labels:
+                all_labels.add(single_label)
+
+        # let user choose a list of label on which it will be performed the analysis
+        selected_label = np.array(
+            select_label_menu(all_labels, min_labels=5, max_labels=15)
+        )
+
+        # perform label based analysis on the apk_
+        malware_confidences = {}
+        for apk_ in apk:
+            data = Quark(apk_)
+            all_labels = {}
+            # dictionary containing
+            # key: label
+            # value: list of confidence values
+            # $ print(all_rules["accessibility service"])
+            # > [60, 40, 60, 40, 60, 40]
+
+            for single_rule in tqdm(rules_list):
+                rulepath = os.path.join(rule, single_rule)
+                rule_checker = RuleObject(rulepath)
+
+                # analyse malware only on rules where appears label selected
+                labels = np.array(rule_checker.label)
+                if len(np.intersect1d(labels, selected_label)) == 0:
+                    continue
+
+                # Run the checker
+                data.run(rule_checker)
+                confidence = rule_checker.check_item.count(True) * 20
+                labels = (
+                    rule_checker.label
+                )  # array type, e.g. ['network', 'collection']
+                for single_label in labels:
+                    if single_label in all_labels:
+                        all_labels[single_label].append(confidence)
+                    else:
+                        all_labels[single_label] = [confidence]
+
+            # extrapolate data used to plot radar chart
+            radar_data = {}
+            for _label in selected_label:
+                confidences = np.array(all_labels[_label])
+                # on radar data use the maximum confidence for a certain label
+                radar_data[_label] = np.max(confidences)
+
+            radar_confidence = [value_ for _label, value_ in radar_data.items()]
+            malware_confidences[apk_.split("/")[-1]] = radar_confidence
+
+        show_comparison_graph(
+            title=f"Malicious Actions Comparison Between {len(apk)} Malwares",
+            lables=selected_label,
+            malware_confidences=malware_confidences,
+            font_size=22,
+        )
 
     if label:
         all_labels = {}
@@ -130,11 +217,11 @@ def entry_point(
 
         for single_rule in tqdm(rules_list):
             rulepath = os.path.join(rule, single_rule)
-            rule_checker = QuarkRule(rulepath)
+            rule_checker = RuleObject(rulepath)
             # Run the checker
             data.run(rule_checker)
             confidence = rule_checker.check_item.count(True) * 20
-            labels = rule_checker._label  # array type, e.g. ['network', 'collection']
+            labels = rule_checker.label  # array type, e.g. ['network', 'collection']
             for single_label in labels:
                 if single_label in all_labels:
                     all_labels[single_label].append(confidence)
@@ -142,15 +229,13 @@ def entry_point(
                     all_labels[single_label] = [confidence]
 
         # get how many label with max confidence >= 80%
-        counter_high_confidence = 0
-        for single_label in all_labels:
-            if max(all_labels[single_label]) >= 80:
-                counter_high_confidence += 1
+        counter_high_confidence = sum(
+            max(value) >= 80 for single_label, value in all_labels.items()
+        )
 
-        print_info("Total Label found: " + yellow(str(len(all_labels))))
+        print_info(f"Total Label found: {yellow(len(all_labels))}")
         print_info(
-            "Rules with label which max confidence >= 80%: "
-            + yellow(str(counter_high_confidence))
+            f"Rules with label which max confidence >= 80%: {yellow(counter_high_confidence)}"
         )
 
         data.show_label_report(rule, all_labels, label)
@@ -169,12 +254,11 @@ def entry_point(
 
         for single_rule in tqdm(rules_list):
             rulepath = os.path.join(rule, single_rule)
-            rule_checker = QuarkRule(rulepath)
+            rule_checker = RuleObject(rulepath)
 
-            labels = rule_checker._label
-            if label_flag:
-                if summary not in labels:
-                    continue
+            labels = rule_checker.label
+            if label_flag and summary not in labels:
+                continue
 
             # Run the checker
             data.run(rule_checker)
@@ -183,7 +267,7 @@ def entry_point(
 
         w = Weight(data.quark_analysis.score_sum, data.quark_analysis.weight_sum)
         print_warning(w.calculate())
-        print_info("Total Score: " + str(data.quark_analysis.score_sum))
+        print_info(f"Total Score: {data.quark_analysis.score_sum}")
         print(data.quark_analysis.summary_report_table)
 
         if classification:
@@ -204,18 +288,17 @@ def entry_point(
 
         for single_rule in tqdm(rules_list):
             rulepath = os.path.join(rule, single_rule)
-            rule_checker = QuarkRule(rulepath)
+            rule_checker = RuleObject(rulepath)
 
-            labels = rule_checker._label
-            if label_flag:
-                if detail not in labels:
-                    continue
+            labels = rule_checker.label
+            if label_flag and detail not in labels:
+                continue
 
             # Run the checker
             data.run(rule_checker)
 
-            print("Rulepath: " + rulepath)
-            print("Rule crime: " + rule_checker._crime)
+            print(f"Rulepath: {rulepath}")
+            print(f"Rule crime: {rule_checker.crime}")
             data.show_detail_report(rule_checker)
             print_success("OK")
 
@@ -229,7 +312,7 @@ def entry_point(
 
         for single_rule in tqdm(rules_list):
             rulepath = os.path.join(rule, single_rule)
-            rule_checker = QuarkRule(rulepath)
+            rule_checker = RuleObject(rulepath)
 
             # Run the checker
             data.run(rule_checker)
