@@ -2,13 +2,16 @@
 # This file is part of Quark-Engine - https://github.com/quark-engine/quark-engine
 # See the file 'LICENSE' for copying permission.
 
+import plotly.graph_objects as go
+import sys
 from graphviz import Digraph
+from prompt_toolkit.shortcuts import checkboxlist_dialog
 
 
-def wrapper_lookup(wrapper, top_method, native_api):
+def wrapper_lookup(apkinfo, wrapper, top_method, native_api):
     next_level = []
 
-    for _, method, _ in top_method.get_xref_to():
+    for method, _ in apkinfo.lowerfunc(top_method):
         if method == native_api:
             wrapper.append(top_method)
             return
@@ -18,7 +21,7 @@ def wrapper_lookup(wrapper, top_method, native_api):
             next_level.append(method)
 
     for next_level_method in next_level:
-        wrapper_lookup(wrapper, next_level_method, native_api)
+        wrapper_lookup(apkinfo, wrapper, next_level_method, native_api)
 
 
 def call_graph(call_graph_analysis):
@@ -27,6 +30,7 @@ def call_graph(call_graph_analysis):
     """
 
     parent_function = call_graph_analysis["parent"]
+    apkinfo = call_graph_analysis["apkinfo"]
     first_call = call_graph_analysis["first_call"]
     second_call = call_graph_analysis["second_call"]
     first_api = call_graph_analysis["first_api"]
@@ -37,9 +41,9 @@ def call_graph(call_graph_analysis):
     second_wrapper = []
 
     if first_call != first_api:
-        wrapper_lookup(first_wrapper, first_call, first_api)
+        wrapper_lookup(apkinfo, first_wrapper, first_call, first_api)
     if second_call != second_api:
-        wrapper_lookup(second_wrapper, second_call, second_api)
+        wrapper_lookup(apkinfo, second_wrapper, second_call, second_api)
 
     # Initialize the Digraph object
     dot = Digraph(
@@ -71,7 +75,7 @@ def call_graph(call_graph_analysis):
         p, r = str(parent_function.descriptor).split(")")
         mutual_parent_function_description.node(
             f"{parent_function.full_name}",
-            label=f"Access: {parent_function.access}\nClass: {parent_function.class_name}\nMethod: {parent_function.name}\n Parameter: {p})\n Return: {r}",
+            label=f"Access: {parent_function.access_flags}\nClass: {parent_function.class_name}\nMethod: {parent_function.name}\n Parameter: {p})\n Return: {r}",
             shape="none",
             fontcolor="blue",
             fontname="Courier New",
@@ -89,7 +93,7 @@ def call_graph(call_graph_analysis):
 
                 wrapper.node(
                     f"{wp_func.full_name}",
-                    label=f"Access: {wp_func.access}\nClass: {wp_func.class_name}\nMethod: {wp_func.name}\n Parameter: {p})\n Return: {r}",
+                    label=f"Access: {wp_func.access_flags}\nClass: {wp_func.class_name}\nMethod: {wp_func.name}\n Parameter: {p})\n Return: {r}",
                     style="rounded",
                     fontcolor="blue",
                     penwidth="1",
@@ -113,7 +117,7 @@ def call_graph(call_graph_analysis):
                 p, r = str(wp_func.descriptor).split(")")
                 wrapper.node(
                     f"{wp_func.full_name}",
-                    label=f"Access: {wp_func.access}\nClass: {wp_func.class_name}\nMethod: {wp_func.name}\n Parameter: {p})\n Return: {r}",
+                    label=f"Access: {wp_func.access_flags}\nClass: {wp_func.class_name}\nMethod: {wp_func.name}\n Parameter: {p})\n Return: {r}",
                     style="rounded",
                     fontcolor="blue",
                     penwidth="1",
@@ -185,7 +189,6 @@ def call_graph(call_graph_analysis):
     if second_call != second_api:
         dot.edge(
             f"{parent_function.full_name}",
-
             f"{second_wrapper[-1].full_name}",
             "Second Call",
             fontname="Courier New",
@@ -205,4 +208,76 @@ def call_graph(call_graph_analysis):
             fontname="Courier New",
         )
 
-    dot.render(f"call_graph_image/{parent_function.name}_{first_call.name}_{second_call.name}")
+    dot.render(
+        f"call_graph_image/{parent_function.name}_{first_call.name}_{second_call.name}"
+    )
+
+
+def show_comparison_graph(title, lables, malware_confidences, font_size=22):
+    """
+    show radar chart based on max label confidence of several malwares
+    :param title: title of the graph to be displayed
+    :param labels: labels to be shown on the radar chart
+    :param malware_confidences: dictionary with structure, malware_name=[
+    array of confidences to be shown on radar chart]
+    :return: None
+    """
+    fig = go.Figure()
+    # plot the graph with specific layout
+    fig.update_layout(
+        polar=dict(radialaxis=dict(visible=True, range=[0, 100], dtick=20)),
+        showlegend=True,
+        title={
+            "text": f"<b>{title}</b>",
+        },
+        font=dict(size=font_size),
+        title_x=0.5,
+        legend=dict(
+            y=0.5,
+            x=0.8,
+            traceorder="normal",
+        ),
+    )
+    for malware_name in malware_confidences:
+        fig.add_trace(
+            go.Scatterpolar(
+                r=malware_confidences[malware_name],
+                theta=lables,
+                fill="toself",
+                name=malware_name,
+                line=dict(
+                    width=4,
+                ),
+            )
+        )
+    fig.show()
+
+
+def select_label_menu(all_labels, min_labels=5, max_labels=10):
+    """
+    allows user to select label to be shown in radar chart
+    :param all_labels: all label found on the rules
+    :param min_labels: min label to be shown on radar chart (default 5)
+    :param max_labels: max label to be shown on radar chart (default 10)
+    :return: label selected
+    """
+
+    value_pair = [(label, label) for label in all_labels]
+
+    while True:
+        results_array = checkboxlist_dialog(
+            title="Label-base Report",
+            text=f"Select number of labels between {min_labels} and {max_labels}",
+            values=value_pair,
+        ).run()
+
+        if results_array:
+
+            if min_labels <= len(results_array) <= max_labels:
+                break
+        else:
+            # user selects "Cancel" to leave the program.
+            print("Canceled!")
+            sys.exit()
+
+    return results_array
